<?php /*
 * Plugin Name: Alfasearch Langs
 * Plugin URI:  ###
 * Description: Creation of multilingual pages for Wordpress
 * Version: 1.1.1
 * Author: Alfasearch WebDev team
 * Author URI: 
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 *
 * Text Domain: languages
 * Domain Path: /languages
 *
 * Network: true
 */

 defined( 'ABSPATH' ) || exit;

 final class AlfaLang {

    protected static $instance = null;

    protected $post_id = null;

    public $lang_id = null;

    public $lang_slug = null;

    public $lang_flag = null;

    public $lang_name = null;

    public $lang_attr = null;

    protected $menu_id = null;

    public $amp = false;

    public $main_link = null;

    public $brand_name = null;

    public $log_btn = null;

    public $log_btn_text = null;

    public $reg_btn = null;

    public $reg_btn_text = null;

    public $parent_post_id = null;

    public $related = null;

    public $post_object = null; 

    public $main_lang_id = null;
     
    public $related_ids = null;

    public $lang_categories = [];

    /**
	 * Retrieve main AlfaLang instance.
	 *
	 * Ensure only one instance is loaded or can be loaded.
	 *
	 * @see asl()
	 * @return AlfaLang
	 */

	public static function get() {
		if ( is_null( self::$instance ) && ! ( self::$instance instanceof AlfaLang ) ) {
			self::$instance = new AlfaLang();
			self::$instance->setup();
		}

		return self::$instance;
	}

    /**
	 * Instantiate the plugin.
	 */
	private function setup() {
		
        $this->define_constants();
		$this->init_actions();

	}

    public function instantiate_data(){

        $this->post_id = get_the_ID();
        $this->amp = $_GET['amp'] ? true : false;

        /** GENERAL FIELDS **/

        $this->lang_id = get_the_category( $this->post_id )[0]->term_id;

        if ( is_404() )  $this->lang_id = get_field( 'main_version_404', 'option' );

        $this->parent_post_id = get_field( 'main_version_link', $this->post_id );
        $this->main_link = get_option('options_mainLink');
        $this->brand_name = get_field('brand_name', 'options');
        
        /** PAGES **/
        
        $this->related_ids = get_field( 'hreflangs__x-default', $this->post_id );
        
        /** CATEGORY FIELDS **/

        $this->lang_slug = get_field( 'lang_slug', 'category_'. $this->lang_id );
        $this->lang_name = get_field( 'lang_name', 'category_'. $this->lang_id );
        $this->lang_attr = get_field( 'lang_attr', 'category_'. $this->lang_id );
        $this->lang_flag = get_field( 'lang_flag', 'category_'. $this->lang_id )['url'];
        $this->menu_id = get_field( 'chosen_menu_top', 'category_'. $this->lang_id );
        $this->log_btn_text = get_field('button_log_translate', 'category_' . $this->lang_id );
        $this->reg_btn_text = get_field('button_enter_translate', 'category_' . $this->lang_id );
        $this->related = $this->get_related_pages( $this->parent_post_id );
        $this->post_object = get_post($this->post_id);
        $this->main_lang_id = get_field( 'main_version_name', 'options' );
        

    }

    private function init_actions() {

       /* Back-end actions/filters  */
			
        add_action( 'init', [ $this, 'add_taxonomy_support' ], 5 );
        add_action( 'init', [ $this, 'modify_taxonomy' ], 11 );
        add_action( 'init', [$this, 'alfa_add_script'] );
        add_action( 'rest_page_query', [ $this, 'test_rest_page_query' ], 10, 2 );
        add_action( 'acf/init', [ $this, 'acf_import_json' ] );
        add_action( 'wp', [ $this, 'instantiate_data' ], 15 );
        add_action( 'manage_pages_custom_column', [ $this, 'custom_columns' ], 10, 1 );
        add_action( 'admin_menu', [ $this, 'create_languages_table' ], 10 );
        add_action( 'admin_menu', [ $this, 'add_menu_pages' ], 20 );
        add_action( 'wp_ajax_save_data', [$this, 'save_languages_data'] );
        add_filter( 'manage_pages_columns', [ $this, 'add_new_columns' ]);
        add_filter( 'rank_math/opengraph/facebook/og_locale', [ $this, 'change_rank_math_locale' ]);
        add_filter( 'rank_math/frontend/breadcrumb/items', [ $this, 'change_rank_math_breadcrumbs'], 10, 2);

       /* Front-end actions/filters  */

        add_action( 'wp_head', [ $this, 'hreflang_generate'] );
        add_action( 'alfa_htmlLang', [ $this, 'get_attr_lang'] );
        add_action( 'alfa_logo', [ $this, 'get_logo_url'] );
        add_action( 'alfa_nav', [ $this, 'get_langs_choose'] );
        add_action( 'alfa_menu', [ $this, 'get_menu_links'], 10, 1 );
        add_action( 'alfa_buttons', [ $this, 'get_button'],10, 3 );

    }

    //Добавляем поддержку таксономий для страниц
    public function add_taxonomy_support() {  

        register_taxonomy_for_object_type('post_tag', 'page'); 
        register_taxonomy_for_object_type('category', 'page');  

    }

    public function create_languages_table() {
        
        $sql = 'CREATE TABLE alfa_translations(
            id int(10) NOT null AUTO_INCREMENT, 
            original TINYTEXT NOT null,
            PRIMARY KEY (id)
            )'; 

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        dbDelta($sql);
        $this->create_languages_columns();
    }

    public function create_languages_columns() {

        $categories = get_categories(array(
            'orderby'    => 'term_id',
            'hide_empty' => false
        ));

        global $wpdb;

        foreach ($categories as $category) {
            if (get_field( 'lang_slug', 'category_' . $category->term_id )) {
                $lang_slug = get_field( 'lang_slug', 'category_' . $category->term_id );
                $lang_slug_no_hyphen = str_replace('-', '_', $lang_slug);
                $sql = $wpdb->prepare('SELECT COLUMN_NAME
                        FROM information_schema.COLUMNS
                        WHERE TABLE_NAME = "alfa_translations"
                        AND COLUMN_NAME = "%s" ', $lang_slug_no_hyphen);
                $result = $wpdb->get_results($sql);
                if (empty($result)) {
                    $table_sql = 'ALTER TABLE alfa_translations ADD '. $lang_slug_no_hyphen .' TINYTEXT NOT null';
                    $wpdb->query($table_sql);
                }
                $this->lang_categories[$lang_slug_no_hyphen] = $category->name;
           }
        }  

    }

    public function add_menu_pages() {

        add_menu_page('Alfa Langs', 'Alfa Langs', 'manage_options', 'alfa-options', [$this, 'show_languages_options'], 'dashicons-translation');
        add_submenu_page('alfa-options', 'Перевод строк', 'Перевод строк', 'manage_options', 'alfa-translate', [$this, 'show_languages_table']);

    }

    public function show_languages_options() {
        echo 'Hello 1';
    }


    public function alfa_add_script() {
        
        if ( is_admin() ) {
            wp_enqueue_script( 'custom_js', '/wp-content/plugins/alfasearch-langs/js/index.js' );  
            wp_enqueue_style( 'custom_css', '/wp-content/plugins/alfasearch-langs/css/index.css' );
        } 

    }

    public function get_languages_data() {

        global $wpdb;

        $lang_keys = implode(',', array_keys($this->lang_categories));
        
        return $wpdb->get_results("SELECT id,original,{$lang_keys} FROM alfa_translations");

    }


    public function show_languages_table() {

        require_once 'templates/languages-table.php';

        echo $result;

    }

    public function save_languages_data() {

        global $wpdb;

        $field = $_POST['field']; 

        $id = intval($_POST['id']);

        $value = $_POST['value'];

        $result = $wpdb->get_results($wpdb->prepare("SELECT * FROM alfa_translations WHERE id = %d", $id));

        if (empty($result)) {
            $sql = 'INSERT INTO alfa_translations (id, '. $field .') VALUES ('. $id .', "'. $value .'") ';
        } else {
            $sql = 'UPDATE alfa_translations SET '. $field.'="'. $value .'" WHERE id='. $id .'  ';
        }

        $wpdb->query($sql);

        die();

    }


    public function translate($original) {

        global $wpdb;

        $slug = str_replace('-', '_', $this->lang_slug);
        $sql = 'SELECT '. $slug .' FROM alfa_translations WHERE original="'. $original .'"';
        $row = $wpdb->get_row($sql);

        if (empty($row) || $row->$slug == '') return $original;

        return $row->$slug;

    }

    // HELPER

    public function dd($data) {
        echo '<pre>'; 
        print_r($data); 
        echo '</pre>';
    }


    //Вывод кнопок
    public function get_button($text, $split, $buttonClass){
		
	   $event = $this->amp 
	   ? 'on="tap:AMP.navigateTo(url=\''.$this->main_link.'&split='.$split.'\')"' 
	   : 'onclick="location.href=\''.$this->main_link.'&split='.$split.'\'"';

        echo '<button class="button '.$buttonClass.'" '. $event .'>'. $text .'</button>';      
        
    }


    public function test_rest_page_query( $args, $request ) {
    
        $args['post_parent'] = 0 ;
        return $args;
    
    }

    //Создаем аттрибуты hreflang

    public function hreflang_generate(){

        $args = array( 'parent' => 0, 'hide_empty' => 0 );

        $categories = get_categories( $args );
        
        // page has parent id

        if (  $this->parent_post_id ):
            
            foreach( $this->related as $r ):
            
                $language_id = get_the_category($r->ID)[0]->term_id;
                
                $language_slug = get_field( 'lang_attr', 'category_' . $language_id );
                
                $amp_postfix = '';
                        
                if ( $this->amp ): $amp_postfix = '?amp=1'; endif;
                
                //Создаем тег link для всех страниц кроме основной версии
                if ( $r->ID != $this->parent_post_id  ):
                    
                    echo '<link rel="alternate" hreflang="'. $language_slug .'" href="'. get_the_permalink($r->ID) . $amp_postfix .'">' . PHP_EOL;
                    
                        if ( strpos($language_slug, '-') !== false ):
                    
                            $language_slug_stripped = strstr($language_slug, '-', true);
                    
                            echo '<link rel="alternate" hreflang="'. $language_slug_stripped .'" href="'. get_the_permalink($r->ID) . $amp_postfix .'">' . PHP_EOL;
                    
                        endif;
                    
                endif;
                
            endforeach;
        
        // page hasn't parent id, but has related ids' list
        
        elseif ( !$this->parent_post_id && $this->related_ids ):
        
        	$i = 0;
        
        	foreach( $this->related_ids as $rel_id ):
        
         		$language_id = get_the_category($rel_id)[0]->term_id;
                
                $language_slug = get_field( 'lang_attr', 'category_' . $language_id );
                
                $amp_postfix = '';
                        
                if ( $this->amp ): $amp_postfix = '?amp=1'; endif;
        
        		// Первым элементом всегда должен быть x-default
        
        		if ( $i == 0 && ( count($categories) > 1 ) ):
        
        			echo '<link rel="alternate" hreflang="x-default" href="'. get_the_permalink($rel_id) . $amp_postfix .'">' . PHP_EOL;
        
        		endif;
        		
        		// Соседи по несчастью
        
        			echo '<link rel="alternate" hreflang="'. $language_slug .'" href="'. get_the_permalink($rel_id) . $amp_postfix .'">' . PHP_EOL;
                    
                        if ( strpos($language_slug, '-') !== false ):
                    
                            $language_slug_stripped = strstr($language_slug, '-', true);
                    
                            echo '<link rel="alternate" hreflang="'. $language_slug_stripped .'" href="'. get_the_permalink($rel_id) . $amp_postfix .'">' . PHP_EOL;
                    
                        endif;
        	
        		$i++;
        
        	endforeach;

        endif;
            
            //И для основной версии
        
        if( !is_404() &&  $this->parent_post_id && !$this->related_ids && ( count($categories) > 1 ) ):
        
            echo '<link rel="alternate" hreflang="x-default" href="'. get_the_permalink($this->parent_post_id) . $amp_postfix .'">' . PHP_EOL;
        
        // Одиночка - страница без основной версии и без соседей
        
        elseif( !is_404() &&  !$this->parent_post_id && !$this->related_ids && ( count($categories) > 1 ) ):
        
        	echo '<link rel="alternate" hreflang="x-default" href="'. get_the_permalink($this->post_id) . $amp_postfix .'">' . PHP_EOL;
        
        
        endif;	
            
    }    

    public function change_rank_math_locale(){
	
	    return $this->lang_attr;

    }

    public function change_rank_math_breadcrumbs($crumbs, $class) {

        $crumbs[0][1] = $this->get_slug_home_url();

        return $crumbs;

    }
      
    public function get_related_pages( $parent_post_id ){

        // Статьи, у которых основная версия одинаковая
        $related = get_posts(array(
            'numberposts'	=> -1,
            'post_type'		=> 'page',
            'status'		=> 'publish',
            'meta_query' => array(
                array(
                   'key'     => 'main_version_link',
                   'value'   => $parent_post_id,
                )
             )
        ));

       for( $i = 0; $i < count($related); $i++ ){

            if ( !get_field('main_version_link', $related[$i]->ID)  ) unset($related[$i]);

        }
        
        return $related;
        
    }
    
    //убрать создание страниц категории
    public function modify_taxonomy() {
        
        $category_args = get_taxonomy( 'category' ); 
		$menu_args = get_taxonomy('nav_menu'); 

        $category_args->publicly_queryable = false;
        $category_args->query_var = false;
		
		$menu_args->show_in_rest = true;
		$menu_args->public = true;
    
        register_taxonomy( 'category', array('post', 'page'), (array) $category_args );
		register_taxonomy( 'nav_menu', array('nav_menu_item'), (array) $menu_args );
    
    }

    /**
    * Add new columns to the post table
    *
    * @param Array $columns - Current columns on the list post
    */
    public function add_new_columns($columns){

        $column_meta = array( 'linked' => 'Связанные страницы' );
        $columns = array_slice( $columns, 0, 4, true ) + $column_meta + array_slice( $columns, 4, NULL, true );
        return $columns;

    }

    /**
    * Display data in new columns
    *
    * @param  $column Current column
    *
    * @return Data for the column
    */
    public function custom_columns($column) {

        global $post;

        switch ( $column ) {
            
            case 'linked':

                $main_version_id = get_field( 'main_version_link', $post->ID );

                $related= $this->get_related_pages( $main_version_id  );

                if ( $related ) {
                    
                    foreach ($related as $r){

                        if ( $post->ID != $r->ID ):

                            $cat_id = get_the_category( $r->ID )[0]->term_id;

                            $flag = get_field('lang_flag', 'category_' . $cat_id)['url'];

                            echo '<p style="display:flex;align-items:center;">
                            <img style="max-width:16px;margin-right:8px;" src="'.  $flag .'" alt="" />
                            <a href="/wp-admin/post.php?post='. $r->ID .'&action=edit" target="_blank">'. $r->post_title .'</a>
                            </p>';

                        endif;

                    }

                }     

            break;
        }
    }

    //url для логотипа
    public function get_logo_url(){

        $menu = wp_get_nav_menu_items( $this->menu_id );

        $counter = 0;
        
        foreach( $menu as $item ):
        
            if ($counter == 0):
        
                echo $item->url; 	

                break;
            
            endif;
            
        $counter++;
        
        endforeach;
        
    }	


    public function acf_import_json(){

		if (function_exists('acf_import_field_group')) {
			
			$acf_json_data = ASL_PLUGIN_DIR . 'acf.json';

			$custom_fields = $acf_json_data ? json_decode(file_get_contents($acf_json_data), true) : array();
	
			foreach ($custom_fields as $custom_field) {
				if (!acf_get_field_group($custom_field['key'])){
					acf_import_field_group($custom_field);
				}	
			}

		}

	}

      /**
	 * Define the plugin constants.
	 */
	private function define_constants() {
		// Plugin DIR, with trailing slash:
        if( !defined('ASL_PLUGIN_DIR') ) {
            define('ASL_PLUGIN_DIR', plugin_dir_path(__FILE__));
        }
	}

    //выводит пункты меню в виде ссылок

    public function get_menu_links($menu_type){
        
        $this->menu_id = get_field('chosen_menu_'. $menu_type .'', 'category_'. $this->lang_id );

        $menu = wp_nav_menu(array(
            'menu' => $this->menu_id,
			'container' => false,
			'echo' => false,
			//'items_wrap' => '<ul>%3$s</ul>',
        ));
		
		
		//if ( !$this->amp ) 
			//$menu = str_replace(
				//array('<ul class="sub-menu">', '</ul></li>'),
				//array('<div class="sub-menu-wrap"><ul class="sub-menu">', '</ul></div></li>'),
				//$menu
			//);
		
		echo $menu;

    }

    //выводим выбор языков в шапке
    public function get_langs_choose(){
        
        // if $this->lang_name 
        if ( $this->lang_name &&  $this->parent_post_id ):
            
            if ( $this->amp ) :
                
                echo '<div class="choose">
                <div class="toggled" [class]="up ? \'toggled\' : \'arrowed\'" on="tap:AMP.setState({option: !option, up: !up})">
                    <img src="'. $this->lang_flag .'" alt="'. $this->lang_name .'">
                    <span>'. $this->lang_name .'</span>
                </div>
                <div class="submenu hide" [class]="option ? \'submenu show\' : \'submenu hide\'">';
            
            //if not amp        
            else:		
            		
                echo '<div class="choose">
                    <img src="'. $this->lang_flag .'" alt="'. $this->lang_name .'">
                    <span>'. $this->lang_name .'</span>
                    <div class="submenu">';

            endif;	
                        
                foreach( $this->related as $r ):
                
                    $cat_id = get_the_category( $r->ID )[0]->term_id;

                    $attr = get_field('lang_attr', 'category_' . $cat_id );
                    $flag = get_field('lang_flag', 'category_' . $cat_id)['url'];
                    $name = get_field('lang_name', 'category_' . $cat_id);
                    $slug = get_field('lang_slug', 'category_' . $cat_id);
                    $link = get_the_permalink( $r->ID );

                    if ( $this->amp ) : 

                        $link = str_replace('https://', '', $link);
                        $link = str_replace('http://', '', $link);
                        $link = 'https://www.google.com/amp/s/' . $link;
                        $link .= '%3Famp=1';
            
                    endif;
                            
                    if ( $this->lang_id != $cat_id ):
                
                        echo '<div data-attr="'. $attr .'" data-slug="'. $slug .'">
                                <a href="'. $link .'">
                                <img src="'. $flag .'" alt="'. $name .'">
                                '. $name .'</a>
                        </div>';
                    
                    endif;

                
                endforeach;

                
            echo '</div></div>';
        
        elseif( $this->lang_name &&  !$this->parent_post_id && $this->related_ids ):
        
        	if ( $this->amp ) :
                
                echo '<div class="choose">
                <div class="toggled" [class]="up ? \'toggled\' : \'arrowed\'" on="tap:AMP.setState({option: !option, up: !up})">
                    <img src="'. $this->lang_flag .'" alt="'. $this->lang_name .'">
                    <span>'. $this->lang_name .'</span>
                </div>
                <div class="submenu hide" [class]="option ? \'submenu show\' : \'submenu hide\'">';
            
            //if not amp        
            else:		
            		
                echo '<div class="choose">
                    <img src="'. $this->lang_flag .'" alt="'. $this->lang_name .'">
                    <span>'. $this->lang_name .'</span>
                    <div class="submenu">';

            endif;	
        
        	foreach( $this->related_ids as $r_id ):
                
                $cat_id = get_the_category( $r_id )[0]->term_id;

                    $attr = get_field('lang_attr', 'category_' . $cat_id );
                    $flag = get_field('lang_flag', 'category_' . $cat_id)['url'];
                    $name = get_field('lang_name', 'category_' . $cat_id);
                    $slug = get_field('lang_slug', 'category_' . $cat_id);
                    $link = get_the_permalink( $r_id );

                    if ( $this->amp ) : 

                        $link = str_replace('https://', '', $link);
                        $link = str_replace('http://', '', $link);
                        $link = 'https://www.google.com/amp/s/' . $link;
                        $link .= '%3Famp=1';
            
                    endif;
                            
                    if ( $this->lang_id != $cat_id ):
                
                        echo '<div data-attr="'. $attr .'" data-slug="'. $slug .'">
                                <a href="'. $link .'">
                                <img src="'. $flag .'" alt="'. $name .'">
                                '. $name .'</a>
                        </div>';
                    
                    endif;
                
                endforeach;
        
         echo '</div></div>';
        
        endif; // if $lang_name && parent_post_id
        
        
    }

    public function get_slug_home_url() {

       if ( is_front_page() || is_404() ) {
            return get_home_url();

       } else {

            if ( $this->post_object->post_parent ) {
                return get_the_permalink($this->post_object->post_parent);

            } else {

                if ( get_the_category($this->post_id)[0]->term_id == $this->main_lang_id ) {
                    return get_home_url();

                } else {
                    return $_SERVER['REQUEST_URI'];
                }
            }       
       }
    
    }

    public function alfa_breadcrumbs(){

        if ( is_front_page() ) return false;
		
		$title = get_the_title( $this->post_id );
		
		if ( get_field('breadcrumbs-text', $this->post_id) != '' ) $title = get_field('breadcrumbs-text', $this->post_id);

        if ($this->post_object->post_parent) {
            return '<a href="' .$this->get_slug_home_url() . '" class="home"> <svg height="20" width="20" fill="#ffffff" viewBox="-3.2 -3.2 38.40 38.40" version="1.1" xmlns="http://www.w3.org/2000/svg" stroke="#ffffff" stroke-width="0.00032">
            <g id="SVGRepo_bgCarrier" stroke-width="0"></g>
            <g id="SVGRepo_tracerCarrier" stroke-linecap="round" stroke-linejoin="round" stroke="#19e141" stroke-width="0.128"></g>
            <g id="SVGRepo_iconCarrier"> 
            <title>home</title> 
            <path d="M19.469 12.594l3.625 3.313c0.438 0.406 0.313 0.719-0.281 0.719h-2.719v8.656c0 0.594-0.5 1.125-1.094 1.125h-4.719v-6.063c0-0.594-0.531-1.125-1.125-1.125h-2.969c-0.594 0-1.125 0.531-1.125 1.125v6.063h-4.719c-0.594 0-1.125-0.531-1.125-1.125v-8.656h-2.688c-0.594 0-0.719-0.313-0.281-0.719l10.594-9.625c0.438-0.406 1.188-0.406 1.656 0l2.406 2.156v-1.719c0-0.594 0.531-1.125 1.125-1.125h2.344c0.594 0 1.094 0.531 1.094 1.125v5.875z"></path> </g>
            </svg>'. $this->brand_name .'</a>
            
            <span>' . $title .'</span>';
        } else {
            if ( get_the_category($this->post_id)[0]->term_id == $this->main_lang_id ) {
                return '<a href="' .get_home_url(). '" class="home"> <svg height="20" width="20" fill="#ffffff" viewBox="-3.2 -3.2 38.40 38.40" version="1.1" xmlns="http://www.w3.org/2000/svg" stroke="#ffffff" stroke-width="0.00032">
                <g id="SVGRepo_bgCarrier" stroke-width="0"></g>
                <g id="SVGRepo_tracerCarrier" stroke-linecap="round" stroke-linejoin="round" stroke="#19e141" stroke-width="0.128"></g>
                <g id="SVGRepo_iconCarrier"> 
                <title>home</title> 
                <path d="M19.469 12.594l3.625 3.313c0.438 0.406 0.313 0.719-0.281 0.719h-2.719v8.656c0 0.594-0.5 1.125-1.094 1.125h-4.719v-6.063c0-0.594-0.531-1.125-1.125-1.125h-2.969c-0.594 0-1.125 0.531-1.125 1.125v6.063h-4.719c-0.594 0-1.125-0.531-1.125-1.125v-8.656h-2.688c-0.594 0-0.719-0.313-0.281-0.719l10.594-9.625c0.438-0.406 1.188-0.406 1.656 0l2.406 2.156v-1.719c0-0.594 0.531-1.125 1.125-1.125h2.344c0.594 0 1.094 0.531 1.094 1.125v5.875z"></path> </g>
                </svg>'. $this->brand_name .'</a>
                
                <span>' . $title .'</span>';
            } 
        }
 
    }

    

 } // CLASS


/**
 * Returns the main instance of AlfaBundle to prevent the need to use globals.
 *
 * @return AlfaLang
*/

function asl() {
	return AlfaLang::get();
}
	

// Start it.
asl();